<?php

declare(strict_types=1);

namespace Drupal\Tests\vgwort\Functional;

use Drupal\FunctionalTests\Update\UpdatePathTestBase;
use Drupal\Tests\system\Functional\Cache\AssertPageCacheContextsAndTagsTrait;
use Drupal\vgwort\EntityJobMapper;

/**
 * Tests the VG Wort module.
 *
 * @group vgwort
 */
class VGWortUpdateBeta4Test extends UpdatePathTestBase {
  use AssertPageCacheContextsAndTagsTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  protected function setUp(): void {
    parent::setUp();
    \Drupal::service('update.post_update_registry')->registerInvokedUpdates([
      'vgwort_post_update_add_node_to_entity_types',
      'vgwort_post_update_set_test_mode',
      'vgwort_post_update_create_queue',
      'vgwort_post_update_add_legal_rights_to_config',
      'vgwort_post_update_daemon_queue',
    ]);
    // Create some fake data in the map so that vgwort_update_20005() has some
    // to update.
    \Drupal::database()->insert(EntityJobMapper::TABLE)
      ->fields(['entity_type', 'entity_id', 'job_id', 'success_timestamp'])
      ->values(['entity_type' => 'node', 'entity_id' => 1, 'job_id' => 1, 'success_timestamp' => NULL])
      ->values(['entity_type' => 'node', 'entity_id' => 2, 'job_id' => 2, 'success_timestamp' => 100000])
      ->values(['entity_type' => 'node', 'entity_id' => 3, 'job_id' => 3, 'success_timestamp' => NULL])
      ->values(['entity_type' => 'node', 'entity_id' => 4, 'job_id' => 4, 'success_timestamp' => 100000])
      ->values(['entity_type' => 'node', 'entity_id' => 5, 'job_id' => 5, 'success_timestamp' => 100000])
      ->values(['entity_type' => 'node', 'entity_id' => 8, 'job_id' => 6, 'success_timestamp' => NULL])
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  protected function setDatabaseDumpFiles() {
    $this->databaseDumpFiles[] = $this->root . '/core/modules/system/tests/fixtures/update/drupal-9.4.0.filled.standard.php.gz';
    $this->databaseDumpFiles[] = __DIR__ . '/../../fixtures/install_vgwort_for_update_testing_2.0.0-beta4.php';
  }

  /**
   * Tests vgwort_update_20004() and vgwort_update_20005().
   */
  public function testVgWortUpdates(): void {
    $schema = \Drupal::database()->schema();
    $find_primary_key_columns = new \ReflectionMethod(get_class($schema), 'findPrimaryKeyColumns');
    $find_primary_key_columns->setAccessible(TRUE);
    // Ensure install_vgwort_for_update_testing_2.0.0-beta4.php is working.
    $this->assertTrue($schema->tableExists(EntityJobMapper::TABLE));
    $this->assertFalse($schema->fieldExists(EntityJobMapper::TABLE, 'revision_id'));
    $this->assertFalse($schema->fieldExists(EntityJobMapper::TABLE, 'counter_id'));
    $this->assertEquals(['entity_type', 'entity_id'], $find_primary_key_columns->invoke($schema, EntityJobMapper::TABLE));
    // Ensure base field has not been created.
    $this->assertFalse($schema->fieldExists('node_field_data', 'vgwort_counter_suffix'));

    // Run updates and test them.
    $this->runUpdates();
    $schema = \Drupal::database()->schema();
    $this->assertTrue($schema->fieldExists(EntityJobMapper::TABLE, 'revision_id'));
    $this->assertTrue($schema->fieldExists(EntityJobMapper::TABLE, 'counter_id'));
    $this->assertEquals(['entity_type', 'entity_id', 'counter_id'], $find_primary_key_columns->invoke($schema, EntityJobMapper::TABLE));
    $this->assertMapUpdated(1, 'vgzm.123456-0b164728-be64-41b3-b610-16be2ec381ef', NULL);
    $this->assertMapUpdated(2, 'vgzm.123456-636df636-7fb0-4435-b325-37d5e4197154', '3');
    $this->assertMapUpdated(3, 'vgzm.123456-8d55f6ae-f1e0-427d-aa09-9a302434da09', NULL);
    $this->assertMapUpdated(4, 'vgzm.123456-2628d7e4-8e93-4e67-8b00-c773651d7d37', '5');
    $this->assertMapUpdated(8, 'vgzm.123456-ebbf8389-3b6d-4469-b500-e74117a0e0b3', NULL);

    // Ensure a row for an entity that does not exist has been deleted.
    $row = \Drupal::database()->select(EntityJobMapper::TABLE, 'map')
      ->fields('map')
      ->condition('entity_type', 'node')
      ->condition('entity_id', 5)
      ->execute()->fetchObject();
    $this->assertFalse($row);

    // Ensure base field has been created.
    // @see vgwort_update_20006()
    $this->assertTrue($schema->fieldExists('node_field_data', 'vgwort_counter_suffix'));
  }

  /**
   * Ensures the map has the expected values after the update.
   *
   * @param int $nid
   *   The node ID to check the map table values for.
   * @param string $counter_id
   *   The expected counter ID.
   * @param string|null $revision_id
   *   The expected revision ID.
   */
  private function assertMapUpdated(int $nid, string $counter_id, ?string $revision_id): void {
    $row = \Drupal::database()->select(EntityJobMapper::TABLE, 'map')
      ->fields('map', ['counter_id', 'revision_id'])
      ->condition('entity_type', 'node')
      ->condition('entity_id', $nid)
      ->execute()->fetchObject();

    $this->assertSame($counter_id, $row->counter_id, "Counter ID is the expected value for node $nid");
    $this->assertSame($revision_id, $row->revision_id, "Revision ID is the expected value for node $nid");
  }

}
